##############################################################################
# Description
#     This is the event handler file for Deckel Maho DMU 50 and 70 eVolution series Dual-Table 
#     machines with Heidenhain i530 control system 5axis mill
#
# Revisions
#
#   	Date        		Who      		Reason
#	24 aug 2002 	Oleg Chizh	Original release
# $HISTORY$
#
#______________________________________________________________________________
#The following command invokes the debugging mode.
#______________________________________________________________________________

# source [MOM_ask_env_var UGII_CAM_DEBUG_DIR]mom_debug.tcl ;source [MOM_ask_env_var UGII_CAM_DEBUG_DIR]mom_review.tcl ; MOM_set_debug_mode ON

#______________________________________________________________________________
#The following command invokes the warning mode and the default settings.
#______________________________________________________________________________

source "[MOM_ask_env_var UGII_CAM_POST_DIR]ugpost_base.tcl"

#______________________________________________________________________________
#
# Set default kinematics for UGPost to provide events and variables.
#______________________________________________________________________________

set mom_kin_arc_valid_plane				XYZ
set mom_kin_output_unit					MM
set mom_kin_arc_output_mode			FULL_CIRCLE
set mom_kin_machine_type				5_axis_dual_table
set mom_kin_4th_axis_plane				YZ
set mom_kin_5th_axis_plane				ZX

set mom_kin_linearization_tol				0.01
set mom_kin_4th_axis_limit_action			"Retract / Re-Engage"
set mom_kin_5th_axis_limit_action			"Retract / Re-Engage"

#______________________________________________________________________________
#The following file consists of default values set for G & M codes
#______________________________________________________________________________

#********M code declaration

set mom_sys_program_stop_code              	0
set mom_sys_optional_stop_code             		1

set mom_sys_spindle_direction_code(CLW)    	3
set mom_sys_spindle_direction_code(CCLW)   4
set mom_sys_spindle_direction_code(OFF)    	5

set mom_sys_tool_change_code               		6

set mom_sys_coolant_code(MIST)             		7
set mom_sys_coolant_code(FLOOD)            	8
set mom_sys_coolant_code(ON)               		8
set mom_sys_coolant_code(TAP)              		8
set mom_sys_coolant_code(OFF)              		9


set mom_sys_clamp_code(ON)                 		10
set mom_sys_clamp_code(OFF)                		11
set mom_sys_clamp_code(AXISON)             	10
set mom_sys_clamp_code(AXISOFF)            	11

set mom_sys_end_of_program_code            	30

#********G code declaration

set mom_sys_rapid_code                     		0

set mom_sys_linear_code                    			1

set mom_sys_circle_code(CLW)               		2
set mom_sys_circle_code(CCLW)              		3

set mom_sys_delay_code(SECONDS)           	4

set mom_sys_mcs_rotation_code              		7

set mom_sys_cutcom_plane_code(XY)          	17
set mom_sys_cutcom_plane_code(ZX)          	18
set mom_sys_cutcom_plane_code(YZ)          	19

set mom_sys_cutcom_code(OFF)               		0
set mom_sys_cutcom_code(LEFT)              	L
set mom_sys_cutcom_code(RIGHT)             	R

set mom_sys_inch_code                      			70
set mom_sys_metric_code                    		71

set mom_activate_cycle                     			99

set mom_sys_cycle_off                      			80
set mom_sys_cycle_drill_code               		81
set mom_sys_cycle_drill_dwell_code         		81
set mom_sys_cycle_drill_deep_code          	83
set mom_sys_cycle_tap_code                 		84
set mom_sys_cycle_bore_code                		85
set mom_sys_cycle_bore_dwell_code          	85
set mom_sys_cycle_bore_drag_code           	86
set mom_sys_cycle_bore_back_code           	87
set mom_sys_cycle_bore_manual_code         	88
set mom_sys_cycle_bore_manual_dwell_code	89
 
set mom_sys_output_code(ABSOLUTE)          	90
set mom_sys_output_code(INCREMENTAL)	91

set mom_sys_list_output					OFF  
set mom_sys_header_output				OFF  
set mom_sys_list_file_rows				40   
set mom_sys_list_file_columns				30   
set mom_sys_warning_output				OFF  
set mom_sys_group_output				OFF  
set mom_sys_list_file_suffix				lpt  
set mom_list_file_suffix					lpt  
set mom_sys_output_file_suffix			H  
set mom_output_file_suffix				H

#_____________________________________________
# Global Variable Declaration
#_____________________________________________
set prog_start_flag                        				0
set prog_number                            			12345
set mom_sys_list_file_columns              		132
set mom_sys_list_file_rows                 			50
set header_check_status	            			0
set mom_sys_commentary_output              	OFF
set mom_sys_list_output                    			OFF

set opt 								1
#
# some additional setting
set mom_sys_home_pos(0)                			400 
set mom_sys_home_pos(1)                			200  
set mom_sys_home_pos(2)                			220  

set mom_cycle_rapid_to_pos(0)         			0
set mom_cycle_rapid_to_pos(1)        			0
set mom_cycle_rapid_to_pos(2)         			0
set mom_cycle_feed_to_pos(0)          			0
set mom_cycle_feed_to_pos(1)          			0
set mom_cycle_feed_to_pos(2)          			0
set mom_cycle_retract_to_pos(0)       		0
set mom_cycle_retract_to_pos(1)       		0
set mom_cycle_retract_to_pos(2)       		0
set mom_point1_pos(0) 					-100.0
set mom_point1_pos(1) 					-100.0
set mom_point1_pos(2) 					-50.0
set mom_point2_pos(0) 					100.0
set mom_point2_pos(1) 					100.0
set mom_point2_pos(2) 					0.0
set mom_offset_block					0.0
set mom_smooth_path					"OFF"
set mom_tolerance_on_off				"OFF"
set mom_small_steps					"OFF"
set status_block_create					0
set cutcom_on 							0

#_______________________________________________________________________________

proc  MOM_start_of_program {} {
#_______________________________________________________________________________
# This procedure is executed at the very begining of the program.
# It gets called before any command is read from the task file.
#_______________________________________________________________________________
	global mom_part_name
	global mom_logname
	global mom_date
	global mom_spindle_rpm
	global mom_cutcom_mode
	global mom_cutcom_status
	global mom_output_file_basename
	global p_name
	
	set mom_coolant_status UNDEFINED
	set mom_cutcom_status  UNDEFINED
  	set mom_clamp_status   UNDEFINED
  	set mom_cycle_status   UNDEFINED
  	set mom_spindle_status UNDEFINED
  	set mom_cutcom_plane   UNDEFINED
  	
	catch {unset mom_cutcom_adjust_register}
	catch {unset mom_tool_adjust_register}
  	catch {unset mom_tool_length_adjust_register}
  	catch {unset mom_length_comp_register}
  	catch {unset mom_flush_register}
  	catch {unset mom_wire_cutcom_adjust_register}

	set p_name $mom_output_file_basename
	set part_name [GET_PART_NAME $mom_part_name]

# Setup the kinematics for the DMC50 5-Axis machine

	LOAD_5AXIS_KINEMATICS

# The following procedure opens the warning and listing files
	OPEN_files
# The following procedure lists the header information in commentary data
	LIST_FILE_HEADER 

	set mom_spindle_rpm 0.0
# 	MOM_suppress always N

       # MOM_output_literal " $p_name MM"

#	MOM_suppress off N

# Unset the default cutcom mode
#	catch {unset mom_cutcom_mode}
#	catch {unset mom_cutcom_status}
}
#=============================================================
proc MOM_msys {} {
#=============================================================	
	global mom_spindle_speed
	global status_block_create mom_tolerance_on_off
	
	if {$status_block_create == "0"} {
		MOM_block_create
	}
	if {$mom_tolerance_on_off == "ON"} {
  		MOM_do_template tolerances_1
  		MOM_do_template tolerances_2
  	}	
	MOM_spindle_rpm
}
#=============================================================
proc MOM_operator_message {} {
#=============================================================	

     global mom_operator_message

     MOM_output_literal ";$mom_operator_message"
     MOM_start_of_group
}

#=============================================================
proc MOM_end_of_program { } {
#=============================================================
   	global mom_group_name mom_machine_time line1 d_time
   	 
    	MOM_output_literal "M30"
	set line1 [format "; MACHINE TIME = %.2f " $mom_machine_time]
    	MOM_output_literal "$line1"
    	set mom_group_name [string toupper $mom_group_name]
	MOM_output_literal "END PGM $mom_group_name MM"	

  	LIST_FILE_TRAILER
  	
  	CLOSE_files
}

#=============================================================
proc MOM_nurbs_move { } {
#       
#=============================================================
	global mom_nurbs_point_count
	global mom_nurbs_points
	global mom_nurbs_coefficients
	global xe ye ze
	global ii
	global x0 x1 x2
	global y0 y1 y2
	global z0 z1 z2
	global E_string_x0 E_string_x1 E_string_x2 
	global E_string_y0 E_string_y1 E_string_y2
	global E_string_z0 E_string_z1 E_string_z2
	global nurbs_precision comp_value mom_smooth_path

	set E_string_x0 ""
	set E_string_x1 ""
	set E_string_x2 ""
	set E_string_y0 ""
	set E_string_y1 ""
	set E_string_y2 ""
	set E_string_z0 ""
	set E_string_z1 ""
	set E_string_z2 ""
	
	MOM_force once X Y Z K3X K2X K1X K3Y K2Y K1Y K3Z K2Z K1Z
	
	for {set ii 1} {$ii < [expr $mom_nurbs_point_count-1]} {incr ii} {
		set xe $mom_nurbs_points($ii,0) 
		set ye $mom_nurbs_points($ii,1)
		set ze $mom_nurbs_points($ii,2)
		set nurbs_precision 1.0
		
		set x0 [expr $mom_nurbs_coefficients($ii,0)*$nurbs_precision]
		set x1 [expr $mom_nurbs_coefficients($ii,1)*$nurbs_precision]
		set x2 [expr $mom_nurbs_coefficients($ii,2)*$nurbs_precision]
		set y0 [expr $mom_nurbs_coefficients($ii,3)*$nurbs_precision]
		set y1 [expr $mom_nurbs_coefficients($ii,4)*$nurbs_precision]
		set y2 [expr $mom_nurbs_coefficients($ii,5)*$nurbs_precision]
		set z0 [expr $mom_nurbs_coefficients($ii,6)*$nurbs_precision]
		set z1 [expr $mom_nurbs_coefficients($ii,7)*$nurbs_precision]
		set z2 [expr $mom_nurbs_coefficients($ii,8)*$nurbs_precision]

		set comp_value [expr abs($x0)]
		if { $comp_value  >= 10.0 } { 
			set x0 [expr $x0/10.0] ; set E_string_x0 "E1"	
		} elseif { $comp_value >= 100.0 } {
			set x0 [expr $x0/100.0] ; set E_string_x0 "E2"
		} elseif { $comp_value >=  1000.0 } {
			set x0 [expr $x0/1000.0] ; set E_string_x0 "E3"
		} else { set E_string_x0 ""}

		set comp_value [expr abs($x1)]
		if { $comp_value >=  10.0} { 
			set x1 [expr $x1/10.0] ; set E_string_x1 "E1" 
		} elseif { $comp_value >=  100.0} {
			set x1 [expr $x1/100.0] ; set E_string_x1 "E2"
		} elseif { $comp_value >=  1000.0} {
			set x1 [expr $x1/1000.0] ; set E_string_x1 "E3"
		} else {set E_string_x1 ""}

		set comp_value [expr abs($x2)]	
		if {$comp_value >=  10.0} { 
			set x2 [expr $x2/10.0] ; set E_string_x2 "E1" 
		} elseif { $comp_value >=  100.0} { 
			set x2 [expr $x2/100.0] ; set E_string_x2 "E2"
		} elseif { $comp_value >=  1000.0} {
			set x2 [expr $x2/1000.0] ; set E_string_x2 "E3"
		} else {set E_string_x2 ""}

		set comp_value [expr abs($y0)]
		if {$comp_value >=  10.0} { 
			set y0 [expr $y0/10.0] ; set E_string_y0 "E1" 
		} elseif { $comp_value >=  100.0} {
			set y0 [expr $y0/100.0] ; set E_string_y0 "E2"
		} elseif { $comp_value >=  1000.0} {
			set y0 [expr $y0/1000.0] ; set E_string_y0 "E3"
		} else {set E_string_y0 ""}

		set comp_value [expr abs($y1)]
		if { $comp_value >=  10.0 } { 
			set y1 [expr $y1/10.0] ; set E_string_y1 "E1" 
		} elseif { $comp_value >=  100.0 } {
			set y1 [expr $y1/100.0] ; set E_string_y1 "E2"
		} elseif { $comp_value >=  1000.0 } {
			set y1 [expr $y1/1000.0] ; set E_string_y1 "E3"
		} else {set E_string_y1 ""}

		set comp_value [expr abs($y2)]
		if { $comp_value >=  10.0 } { 
			set y2 [expr $y2/10.0] ; set E_string_y2 "E1" 
		} elseif { $comp_value >=  100.0 } {
			set y2 [expr $y2/100.0] ; set E_string_y2 "E2"
		} elseif { $comp_value >=  1000.0 } {
			set y2 [expr $y2/1000.0] ; set E_string_y2 "E3"
		} else {set E_string_y2 ""}

		set comp_value [expr abs($z0)]
		if { $comp_value >=  10.0 } { 
			set z0 [expr $z0/10.0] ; set E_string_z0 "E1" 
		} elseif { $comp_value >=  100.0 } {
			set z0 [expr $z0/100.0] ; set E_string_z0 "E2"
		} elseif {$comp_value >=  1000.0} {
			set z0 [expr $z0/1000.0] ; set E_string_z0 "E3"
		} else {set E_string_z0 ""}

		set comp_value [expr abs($z1)]
		if {$comp_value >=  10.0} { 
			set z1 [expr $z1/10.0] ; set E_string_z1 "E1" 
		} elseif {$comp_value >=  100.0} {
			set z1 [expr $z1/100.0] ; set E_string_z1 "E2"
		} elseif {$comp_value >=  1000.0} {
			set z1 [expr $z1/1000.0] ; set E_string_z1 "E3"
		} else {set E_string_z1 ""}
		
		set comp_value [expr abs($z2)]
		if { $comp_value >=  10.0 } { 
			set z2 [expr $z2/10.0] ; set E_string_z2 "E1"
		} elseif {$comp_value >=  100.0} {
			set z2 [expr $z2/100.0] ; set E_string_z2 "E2"
		} elseif {$comp_value >=  1000.0} {
			set z2 [expr $z2/1000.0] ; set E_string_z2 "E3"
		} else {set E_string_z2 ""}

		if {$mom_smooth_path == "ON"} {
			MOM_do_template spline_tnc_m90
		} else  {
			MOM_do_template spline_tnc
		}
#		MOM_do_template spline_tnc
	}
}

#=============================================================
# proc MOM_nurbs_move { } {
#       
#=============================================================
#	global mom_nurbs_point_count
#	global mom_nurbs_points
#	global mom_nurbs_coefficients
#	global xe ye ze
#	global ii
#	global x0 x1 x2
#	global y0 y1 y2
#	global z0 z1 z2
#	global E_string_x0 E_string_x1 E_string_x2 
#	global E_string_y0 E_string_y1 E_string_y2
#	global E_string_z0 E_string_z1 E_string_z2
#	global nurbs_precision
#	global comp_value mom_smooth_path

#	set E_string_x0 ""
#	set E_string_x1 ""
#	set E_string_x2 ""
#	set E_string_y0 ""
#	set E_string_y1 ""
#	set E_string_y2 ""
#	set E_string_z0 ""
#	set E_string_z1 ""
#	set E_string_z2 ""
#	
#	MOM_force once X Y Z K3X K2X K1X K3Y K2Y K1Y K3Z K2Z K1Z
#	
#	for {set ii 1} {$ii < [expr $mom_nurbs_point_count-1]} {incr ii} {
#		set xe $mom_nurbs_points($ii,0) 
#		set ye $mom_nurbs_points($ii,1)
#		set ze $mom_nurbs_points($ii,2)
#		set nurbs_precision 1000
		
#		set x0 [expr $mom_nurbs_coefficients($ii,0)*$nurbs_precision]
#		set x1 [expr $mom_nurbs_coefficients($ii,1)*$nurbs_precision]
#		set x2 [expr $mom_nurbs_coefficients($ii,2)*$nurbs_precision]
#		set y0 [expr $mom_nurbs_coefficients($ii,3)*$nurbs_precision]
#		set y1 [expr $mom_nurbs_coefficients($ii,4)*$nurbs_precision]
#		set y2 [expr $mom_nurbs_coefficients($ii,5)*$nurbs_precision]
#		set z0 [expr $mom_nurbs_coefficients($ii,6)*$nurbs_precision]
#		set z1 [expr $mom_nurbs_coefficients($ii,7)*$nurbs_precision]
#		set z2 [expr $mom_nurbs_coefficients($ii,8)*$nurbs_precision]

#		set comp_value [expr abs($x0)]

#		if { $comp_value  >= 10.0 && $comp_value < 100.0} { 
#			set x0 [expr $x0/10.0] ; set E_string_x0 "E1"	
#		} elseif { $comp_value >= 100.0 && $comp_value < 1000.0} {
#			set x0 [expr $x0/100.0] ; set E_string_x0 "E2"
#		} elseif { $comp_value >=  1000.0 && $comp_value < 10000.0} {
#			set x0 [expr $x0/1000.0] ; set E_string_x0 "E3"
#		} elseif { $comp_value >=  10000.0 && $comp_value < 100000.0} {
#			set x0 [expr $x0/1000.0] ; set E_string_x0 "E4"
#		} else { set E_string_x0 ""}

#		set comp_value [expr abs($x1)]

#		if { $comp_value >=  10.0 && $comp_value < 100.0} { 
#			set x1 [expr $x1/10.0] ; set E_string_x1 "E1" 
#		} elseif { $comp_value >=  100.0 && $comp_value < 1000.0} {
#			set x1 [expr $x1/100.0] ; set E_string_x1 "E2"
#		} elseif { $comp_value >=  1000.0 && $comp_value < 10000.0} {
#			set x1 [expr $x1/1000.0] ; set E_string_x1 "E3"
#		} elseif { $comp_value >=  10000.0 && $comp_value < 100000.0} {
#			set x1 [expr $x1/10000.0] ; set E_string_x1 "E4"
#		} else {
#			set E_string_x1 ""}

#		set comp_value [expr abs($x2)]	


#		if {$comp_value >=  10.0 && $comp_value < 100.0} { 
#			set x2 [expr $x2/10.0] ; set E_string_x2 "E1" 
#		} elseif { $comp_value >=  100.0 && $comp_value < 1000.0} { 
#			set x2 [expr $x2/100.0] ; set E_string_x2 "E2"
#		} elseif { $comp_value >=  1000.0 && $comp_value < 10000.0} {
#			set x2 [expr $x2/1000.0] ; set E_string_x2 "E3"
#		} elseif { $comp_value >=  10000.0 && $comp_value < 100000.0} {
#			set x2 [expr $x2/10000.0] ; set E_string_x2 "E4"			
#		} else {set E_string_x2 ""}

#		set comp_value [expr abs($y0)]

#		if {$comp_value >=  10.0 && $comp_value < 100.0} { 
#			set y0 [expr $y0/10.0] ; set E_string_y0 "E1" 
#		} elseif { $comp_value >=  100.0 && $comp_value < 1000.0} {
#			set y0 [expr $y0/100.0] ; set E_string_y0 "E2"
#		} elseif { $comp_value >=  1000.0 && $comp_value < 10000.0} {
#			set y0 [expr $y0/1000.0] ; set E_string_y0 "E3"
#		} elseif { $comp_value >=  10000.0 && $comp_value < 100000.0} {
#			set y0 [expr $y0/10000.0] ; set E_string_y0 "E4"			
#		} else {set E_string_y0 ""}

#		set comp_value [expr abs($y1)]

#		if { $comp_value >=  10.0 } { 
#			set y1 [expr $y1/10.0] ; set E_string_y1 "E1" 
#		} elseif { $comp_value >=  100.0 } {
#			set y1 [expr $y1/100.0] ; set E_string_y1 "E2"
#		} elseif { $comp_value >=  1000.0 } {
#			set y1 [expr $y1/1000.0] ; set E_string_y1 "E3"
#		} elseif { $comp_value >=  10000.0 && $comp_value < 100000.0} {
#			set y1 [expr $y1/10000.0] ; set E_string_y1 "E4"				
#		} else {set E_string_y1 ""}

#		set comp_value [expr abs($y2)]

#		if { $comp_value >=  10.0  && $comp_value < 100.0} { 
#			set y2 [expr $y2/10.0] ; set E_string_y2 "E1" 
#		} elseif { $comp_value >=  100.0 && $comp_value < 1000.0} {
#			set y2 [expr $y2/100.0] ; set E_string_y2 "E2"
#		} elseif { $comp_value >=  1000.0 && $comp_value < 10000.0} {
#			set y2 [expr $y2/1000.0] ; set E_string_y2 "E3"
#		} elseif { $comp_value >=  10000.0 && $comp_value < 100000.0} {
#			set y2 [expr $y2/10000.0] ; set E_string_y2 "E4"				
#		} else {set E_string_y2 ""}

#		set comp_value [expr abs($z0)]

#		if { $comp_value >=  10.0 && $comp_value < 100.0 } { 
#			set z0 [expr $z0/10.0] ; set E_string_z0 "E1" 
#		} elseif { $comp_value >=  100.0 && $comp_value < 1000.0 } {
#			set z0 [expr $z0/100.0] ; set E_string_z0 "E2"
#		} elseif {$comp_value >=  1000.0 && $comp_value < 10000.0} {
#			set z0 [expr $z0/1000.0] ; set E_string_z0 "E3"
#		} elseif { $comp_value >=  10000.0 && $comp_value < 100000.0} {
#			set z0 [expr $z0/10000.0] ; set E_string_z0 "E4"				
#		} else {set E_string_z0 ""}

#		set comp_value [expr abs($z1)]

#		if {$comp_value >=  10.0 && $comp_value < 100.0} { 
#			set z1 [expr $z1/10.0] ; set E_string_z1 "E1" 
#		} elseif {$comp_value >=  100.0 && $comp_value < 1000.0} {
#			set z1 [expr $z1/100.0] ; set E_string_z1 "E2"
#		} elseif {$comp_value >=  1000.0 && $comp_value < 10000.0} {
#			set z1 [expr $z1/1000.0] ; set E_string_z1 "E3"
#		} elseif { $comp_value >=  10000.0 && $comp_value < 100000.0} {
#			set z1 [expr $z1/10000.0] ; set E_string_z1 "E4"				
#		} else {set E_string_z1 ""}
		
#		set comp_value [expr abs($z2)]

#		if { $comp_value >=  10.0 && $comp_value < 100.0 } { 
#			set z2 [expr $z2/10.0] ; set E_string_z2 "E1"
#		} elseif {$comp_value >=  100.0 && $comp_value < 1000.0} {
#			set z2 [expr $z2/100.0] ; set E_string_z2 "E2"
#		} elseif {$comp_value >=  1000.0 && $comp_value < 10000.0} {
#			set z2 [expr $z2/1000.0] ; set E_string_z2 "E3"
#		} elseif { $comp_value >=  10000.0 && $comp_value < 100000.0} {
#			set z2 [expr $z2/10000.0] ; set E_string_z2 "E4"				
#		} else {set E_string_z2 ""}
#
#		if {$mom_smooth_path == "ON"} {
#			MOM_do_template spline_tnc_m90
#		} else  {
#			MOM_do_template spline_tnc
#		}
#		MOM_do_template spline_tnc
#	}
#}

proc  MOM_start_of_path {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of each path in the task
# It gets called after all the general parameters of the path are loaded
# like the path name and the tool name.
#_______________________________________________________________________________
	global mom_path_name
	global mom_tool_name
	global tcl_platform
  
	if {$tcl_platform(os) == "HP-UX"} {
		set suffix "sl"
	} elseif {$tcl_platform(os) == "windows" || $tcl_platform(os) == "Windows NT"} {
		set suffix "dll"
	} else {
		set suffix "so"
	}

	set uf_library "ugpadvkins.$suffix"
	set text "[MOM_ask_env_var UGII_CAM_AUXILIARY_DIR]$uf_library"
	MOM_force once M_spindle S X Y Z R F Axis_4th Axis_5th
}

proc  MOM_end_of_path {} {
#_______________________________________________________________________________
# This procedure is executed at the end of a path.
#_______________________________________________________________________________
	global mom_path_name
	global first_motion_flag

	set first_motion_flag 0
	MOM_suppress off I J K
}

#=============================================================
proc MOM_start_of_group {} {
#      UG   , 
#    
#=============================================================
	global mom_output_file_basename mom_group_name 
	global header_check_status mom_operation_name
	
	if {[info exists mom_group_name] != 0} {
		set mom_group_name $mom_group_name
	} else {
		set mom_group_name $mom_operation_name
	}
#	set mom_group_name $mom_output_file_basename
	if { $header_check_status == 0} { 
		MOM_output_literal "BEGIN PGM $mom_group_name MM"
		set header_check_status 1
	}       
}

proc MOM_end_of_group {} {

}

proc  MOM_tool_change {} {
#_______________________________________________________________________________
# This procedure is executed at any time there is a tool change.
# It gets called after the tool change data is loaded like the tool number 
# and the register numbers.
#_______________________________________________________________________________
	global mom_tool_name

	GET_TOOL_ATTRIBUTE
}

proc  MOM_first_tool {} {
#_______________________________________________________________________________
# This procedure is executed for the first operation, with a tool, in the task.
#_______________________________________________________________________________
	global mom_tool_name

	GET_TOOL_ATTRIBUTE
}

proc  MOM_initial_move {} {
#_______________________________________________________________________________
# This procedure is executed for the initial move of each operation.  It assumes 
# the tool is moving from a safe position at rapid to the start of the operation.
#_______________________________________________________________________________
	global   mom_coolant_mode
	global   mom_pos_arc_axis
	global   mom_cutcom_mode
	global   first_motion_flag

#	MOM_output_literal "L Z-0.1 R0 F MAX M91 "

	MOM_force once  motion_g G X Y Z M Axis_4th Axis_5th coolant_m T G
	        
	MOM_do_template rapid_bc
	MOM_do_template rapid_xy
	MOM_do_template rapid_z
	set first_motion_flag 1
}

proc PROGRAM_START {} {
	global prog_start_flag
	global prog_number

	if {$prog_start_flag == 0} {
		MOM_suppress always N
		MOM_output_literal "BEGIN PRG $prog_number MM"
		MOM_suppress off    N
		set prog_start_flag 1
	}
}

#=============================================================
proc MOM_block_create { } {
#    2-  3-   ,   
#      
#=============================================================	
	global mom_point1_pos
	global mom_point2_pos
	global status_block_create
	global mom_offset_block
	
	MOM_force once X Y Z
	set mom_point1_pos(0) [expr $mom_point1_pos(0)-$mom_offset_block]
	set mom_point1_pos(1) [expr $mom_point1_pos(1)-$mom_offset_block]
	set mom_point1_pos(2) [expr $mom_point1_pos(2)-$mom_offset_block]
	set mom_point2_pos(0) [expr $mom_point2_pos(0)+$mom_offset_block]
	set mom_point2_pos(1) [expr $mom_point2_pos(1)+$mom_offset_block]
	MOM_do_template start_block
	MOM_do_template end_block
	set status_block_create 1
}

proc MOM_first_move {} {
	global mom_motion_event
	global first_motion_flag
  
# inital safety retract to Z machine reference point...Abs position
#	MOM_output_literal "L Z-0.1 M91"

	catch { MOM_$mom_motion_event }
	set first_motion_flag 1
}

#=============================================================
proc MOM_from_move { } {
#=============================================================
  	global mom_feed_rate mom_feed_rate_per_rev  mom_motion_type mom_kin_max_fpm mom_pos mom_prev_pos

  	COOLANT_SET ; CUTCOM_SET ; SPINDLE_SET ; RAPID_SET ;  MODES_SET
}

proc MOM_gohome_move {} {
	global mom_pos
	global mom_prev_pos 
	global mom_mcs_goto
	
	set mom_pos(0)	$mom_mcs_goto(0)
	set mom_pos(1)	$mom_mcs_goto(1)
	set mom_pos(2)	$mom_mcs_goto(2)
	set mom_pos(3)	0.0
	set mom_pos(4)	0.0
	
	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_bc
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_bc
   	}
}

#=============================================================
proc MOM_linear_move { } {
#=============================================================
	global cutcom_mode mom_cutcom_mode mom_cutcom_status
	global mom_pos mom_prev_pos
	global mom_smooth_path mom_small_steps
	global mom_motion_distance mom_tool_diameter
	global cutcom_on
	
	MOM_do_template feedrate_tnc
	
	if {$mom_small_steps == "ON" 		&& 
		$mom_cutcom_status == "ON"	&&
		$mom_motion_distance <= $mom_tool_diameter} {
		MOM_do_template linear_move_m97
	} elseif {$mom_smooth_path == "ON"} {
		if {$cutcom_on == 1} {
			MOM_do_template linear_move
			set cutcom_on 0
		} else {
			MOM_do_template linear_move_m90
		}	
	} else  {
		MOM_do_template linear_move
	}
}

# proc  MOM_linear_move {} {
#_______________________________________________________________________________
# This procedure is executed for each linear move. It gets called after
# the positioning varables, position and tool axis (X,Y,Z,TAX,TAY,TAZ) are loaded.
#_______________________________________________________________________________
#         MOM_do_template linear
# }

#=============================================================
proc MOM_circular_move { } {
#=============================================================
	global mom_arc_direction
	global mom_arc_axis
	
	if {$mom_arc_axis(2) == "1" || $mom_arc_axis(2) == "-1"} {
		MOM_force once XC YC
		MOM_suppress once ZC
	} elseif {$mom_arc_axis(1) == "1" || $mom_arc_axis(1) == "-1"} {
		 MOM_force once XC ZC
		 MOM_suppress once YC
	} elseif {$mom_arc_axis(0) == "1" || $mom_arc_axis(0) == "-1"} {
		 MOM_force once YC ZC
		 MOM_suppress once XC
	}
	MOM_do_template feedrate_tnc
   	MOM_do_template circle_center
   	if {$mom_arc_axis(2) == "1" || $mom_arc_axis(2) == "-1"} {
		 MOM_force once X Y
		MOM_suppress once Z
	} elseif {$mom_arc_axis(1) == "1" || $mom_arc_axis(1) == "-1"} {
		 MOM_force once X Z		 
		 MOM_suppress once Y
	} elseif {$mom_arc_axis(0) == "1" || $mom_arc_axis(0) == "-1"} {
		 MOM_force once Y Z
		 MOM_suppress once X
	}
   	if { $mom_arc_direction == "CLW"} {
      		MOM_do_template circle_end_cw
   	} elseif { $mom_arc_direction == "CCLW" } {
     		MOM_do_template circle_end_ccw
   	}
}

proc  MOM_rapid_move {} {
#_______________________________________________________________________________
# This procedure is executed for each rapid move. It gets called after
# the positioning varables, position and tool axis (X,Y,Z,TAX,TAY,TAZ) are loaded.
#_______________________________________________________________________________
	global mom_last_z_pos
	global mom_pos mom_prev_pos
	global mom_coolant_mode

	set mom_coolant_status $mom_coolant_mode
	
	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_bc
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_bc
   	}

#	MOM_do_template rapid_xyz
}
 
proc  MOM_before_output {} {
#_______________________________________________________________________________
# This procedure is executed just before a line is about to be output
# to the file. It loads the line into a variable MOM_o_buffer and then calls
# this procedure. When it returns from this procedure, the variable MOM_o_buffer
# is read and written to the file.
#_______________________________________________________________________________
#########The following procedure invokes the listing file with warnings.
         LIST_FILE
}

proc  cycle_set {} {
#_______________________________________________________________________________
# This procedure is executed for each new cycle.  This will make sure all   
# non-modal addresses and Cycle parameters are output that need to be output
# at the start of a new cycle.
#_______________________________________________________________________________
	global mom_cycle_retract_mode
	global g_return

	MOM_force once motion_g X Y Z R cycle_dwell cycle_nodrag cycle_step

	if {$mom_cycle_retract_mode == "MANUAL"} {
		set g_return 99
	} elseif {$mom_cycle_retract_mode == "AUTO"} {
		set g_return 98
	} else {
		set g_return 99
	}
} 

proc  MOM_cycle_plane_change {} {
#_______________________________________________________________________________
# This procedure is executed at the end of cycle motion when there is a cycle 
# move occuring at a higher level than the previous position.
#_______________________________________________________________________________
         MOM_do_template cycle_plane_change
}

#=============================================================
proc MOM_drill { } {
#=============================================================
  	global dwell_at_top
  	global surface_coordinate
  	global mom_cycle_delay_mode
  	global mom_cycle_feed_to
       
 	MOM_force once Q200 Q201 Q206 Q202 Q210 Q203 Q204 Q211
  	
  	set surface_coordinate "0.0"
  	set dwell_at_top "0.0"
  	if {![info exists mom_cycle_delay] || [$mom_cycle_delay_mode == "OFF"]} {
  		set dwell_at_top 0.0
  	} else {
  		set 	dwell_at_top 0.1
  	}
  	MOM_do_template cycle_200
  	MOM_suppress always N
  	MOM_do_template tnc_q200
  	MOM_do_template tnc_q201
  	MOM_do_template tnc_q206
  	set mom_cycle_feed_to [expr abs($mom_cycle_feed_to)]  	
  	MOM_do_template tnc_q202
  	MOM_do_template tnc_q210
  	MOM_do_template tnc_q203
 	MOM_do_template tnc_q204
  	MOM_do_template tnc_q211
	MOM_suppress off    N	
}


#=============================================================
proc  MOM_drill_move { } {
#=============================================================
   	global mom_pos
   	global mom_prev_pos

   	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   	}	
  	MOM_do_template cycle_call 
}

#=============================================================
proc MOM_drill_dwell { } {
#=============================================================
  	global dwell_at_top
  	global surface_coordinate
  	global mom_cycle_delay_mode
  	global mom_cycle_delay
  	global mom_cycle_delay_revs
  	global mom_cycle_feed_to
  	
   	MOM_force once Q200 Q201 Q206 Q202 Q210 Q203 Q204 Q211
  	
  	set surface_coordinate "0.0"
  	set dwell_at_top "0.0"
  	if {![info exists mom_cycle_delay] || [$mom_cycle_delay_mode == "OFF"]} {
  		set dwell_at_top 0.0
  	} else {
		set dwell_at_top $mom_cycle_delay
    }
  	MOM_do_template cycle_200
  	MOM_suppress always N
  	MOM_do_template tnc_q200
  	MOM_do_template tnc_q201
  	MOM_do_template tnc_q206
  	set mom_cycle_feed_to [expr abs($mom_cycle_feed_to)]  	
  	MOM_do_template tnc_q202
  	MOM_do_template tnc_q210
  	MOM_do_template tnc_q203
 	MOM_do_template tnc_q204
  	MOM_do_template tnc_q211
	MOM_suppress off    N	
}

#=============================================================
proc  MOM_drill_dwell_move { } {
#=============================================================
	global mom_pos
   	global mom_prev_pos
   	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   	}	
  	MOM_do_template cycle_call
}

#=============================================================
proc MOM_drill_deep { } {
#=============================================================
	global dwell_at_top
  	global surface_coordinate
  	global mom_cycle_delay_mode
  	global mom_cycle_delay
  	global mom_cycle_cam
  	global mom_cycle_feed_to
  	
  	set mom_cycle_cam "0"
   	
   	MOM_force once Q200 Q201 Q206 Q202 Q210 Q203 Q204 Q211 Q212 Q213 Q205 Q208 Q256
  	
  	set surface_coordinate "0.0"
  	set dwell_at_top "0.0"
  	if {![info exists mom_cycle_delay] || [$mom_cycle_delay_mode == "OFF"]} {
  		set dwell_at_top 0.0
  	} else {
		set 	dwell_at_top $mom_cycle_delay
    	}
  	MOM_do_template cycle_203
  	MOM_suppress always N
  	MOM_do_template tnc_q200
  	MOM_do_template tnc_q201
  	MOM_do_template tnc_q206
  	MOM_do_template tnc_q202_1
  	MOM_do_template tnc_q210
  	MOM_do_template tnc_q203
 	MOM_do_template tnc_q204
 	MOM_do_template tnc_q212
	set mom_cycle_cam $mom_cycle_cam
  	MOM_do_template tnc_q213
  	MOM_do_template tnc_q205
  	MOM_do_template tnc_q211
  	MOM_do_template tnc_q208
	MOM_force once Q256
  	MOM_do_template tnc_q256
  	MOM_suppress off    N	
}
#=============================================================
proc  MOM_drill_deep_move { } {
#=============================================================
   	global mom_pos
   	global mom_prev_pos
   	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   	}	
  	MOM_do_template cycle_call
}
#=============================================================
proc MOM_drill_break_chip { } {
#=============================================================
global dwell_at_top
  	global surface_coordinate
  	global mom_cycle_delay_mode
  	global mom_cycle_delay
  	global mom_cycle_cam
  	global mom_cycle_feed_to
  	
  	set mom_cycle_cam "0"
   	
   	MOM_force once Q200 Q201 Q206 Q202 Q210 Q203 Q204 Q211 Q212 Q213 Q205 Q208 Q256
  	
  	set surface_coordinate "0.0"
  	set dwell_at_top "0.0"
  	if {![info exists mom_cycle_delay] || [$mom_cycle_delay_mode == "OFF"]} {
  		set dwell_at_top 0.0
  	} else {
		set 	dwell_at_top $mom_cycle_delay
    	}
  	MOM_do_template cycle_203
  	MOM_suppress always N
  	MOM_do_template tnc_q200
  	MOM_do_template tnc_q201
  	MOM_do_template tnc_q206
  	MOM_do_template tnc_q202_1
  	MOM_do_template tnc_q210
  	MOM_do_template tnc_q203
 	MOM_do_template tnc_q204
 	MOM_do_template tnc_q212
	set mom_cycle_cam $mom_cycle_cam
  	MOM_do_template tnc_q213
  	MOM_do_template tnc_q205
  	MOM_do_template tnc_q211
  	MOM_do_template tnc_q208
	MOM_force once Q256
  	MOM_do_template tnc_q256
  	MOM_suppress off    N	

}
#=============================================================
proc  MOM_drill_break_chip_move { } {
#=============================================================
   	global mom_pos
   	global mom_prev_pos
   	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   	}	
  	MOM_do_template cycle_call
}
#=============================================================
proc MOM_tap { } {
#=============================================================
global dwell_at_top
  	global surface_coordinate
  	global mom_cycle_delay_mode
  	global mom_cycle_delay
  	global mom_cycle_cam
  	global mom_cycle_feed_to
  	
  	set mom_cycle_cam "0"
   	
   	MOM_force once Q200 Q201 Q206 Q202 Q210 Q203 Q204 Q211 Q212 Q213 Q205 Q208 Q256
  	
  	set surface_coordinate "0.0"
  	set dwell_at_top "0.0"
  	if {![info exists mom_cycle_delay] || [$mom_cycle_delay_mode == "OFF"]} {
  		set dwell_at_top 0.0
  	} else {
		set 	dwell_at_top $mom_cycle_delay
    	}
  	MOM_do_template cycle_203
  	MOM_suppress always N
  	MOM_do_template tnc_q200
  	MOM_do_template tnc_q201
  	MOM_do_template tnc_q206
  	MOM_do_template tnc_q202_1
  	MOM_do_template tnc_q210
  	MOM_do_template tnc_q203
 	MOM_do_template tnc_q204
 	MOM_do_template tnc_q212
	set mom_cycle_cam $mom_cycle_cam
  	MOM_do_template tnc_q213
  	MOM_do_template tnc_q205
  	MOM_do_template tnc_q211
  	MOM_do_template tnc_q208
	MOM_force once Q256
  	MOM_do_template tnc_q256
  	MOM_suppress off    N	

}
#=============================================================
proc  MOM_tap_move { } {
#=============================================================
   	global mom_pos
   	global mom_prev_pos
   	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   	}	
  	MOM_do_template cycle_call
}
#=============================================================
proc MOM_bore { } {
#=============================================================
global dwell_at_top
  	global surface_coordinate
  	global mom_cycle_delay_mode
  	global mom_cycle_delay
  	global mom_cycle_cam
  	global mom_cycle_feed_to
  	
  	set mom_cycle_cam "0"
   	
   	MOM_force once Q200 Q201 Q206 Q202 Q210 Q203 Q204 Q211 Q212 Q213 Q205 Q208 Q256
  	
  	set surface_coordinate "0.0"
  	set dwell_at_top "0.0"
  	if {![info exists mom_cycle_delay] || [$mom_cycle_delay_mode == "OFF"]} {
  		set dwell_at_top 0.0
  	} else {
		set 	dwell_at_top $mom_cycle_delay
    	}
  	MOM_do_template cycle_203
  	MOM_suppress always N
  	MOM_do_template tnc_q200
  	MOM_do_template tnc_q201
  	MOM_do_template tnc_q206
  	MOM_do_template tnc_q202_1
  	MOM_do_template tnc_q210
  	MOM_do_template tnc_q203
 	MOM_do_template tnc_q204
 	MOM_do_template tnc_q212
	set mom_cycle_cam $mom_cycle_cam
  	MOM_do_template tnc_q213
  	MOM_do_template tnc_q205
  	MOM_do_template tnc_q211
  	MOM_do_template tnc_q208
	MOM_force once Q256
  	MOM_do_template tnc_q256
  	MOM_suppress off    N	

}
#=============================================================
proc  MOM_bore_move { } {
#=============================================================
   	global mom_pos
   	global mom_prev_pos
   	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   	}	
  	MOM_do_template cycle_call
}
#=============================================================
proc MOM_bore_drag { } {
#=============================================================
global dwell_at_top
  	global surface_coordinate
  	global mom_cycle_delay_mode
  	global mom_cycle_delay
  	global mom_cycle_cam
  	global mom_cycle_feed_to
  	
  	set mom_cycle_cam "0"
   	
   	MOM_force once Q200 Q201 Q206 Q202 Q210 Q203 Q204 Q211 Q212 Q213 Q205 Q208 Q256
  	
  	set surface_coordinate "0.0"
  	set dwell_at_top "0.0"
  	if {![info exists mom_cycle_delay] || [$mom_cycle_delay_mode == "OFF"]} {
  		set dwell_at_top 0.0
  	} else {
		set 	dwell_at_top $mom_cycle_delay
    	}
  	MOM_do_template cycle_203
  	MOM_suppress always N
  	MOM_do_template tnc_q200
  	MOM_do_template tnc_q201
  	MOM_do_template tnc_q206
  	MOM_do_template tnc_q202_1
  	MOM_do_template tnc_q210
  	MOM_do_template tnc_q203
 	MOM_do_template tnc_q204
 	MOM_do_template tnc_q212
	set mom_cycle_cam $mom_cycle_cam
  	MOM_do_template tnc_q213
  	MOM_do_template tnc_q205
  	MOM_do_template tnc_q211
  	MOM_do_template tnc_q208
	MOM_force once Q256
  	MOM_do_template tnc_q256
  	MOM_suppress off    N	

}
#=============================================================
proc  MOM_bore_drag_move { } {
#=============================================================
   	global mom_pos
   	global mom_prev_pos
   	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   	}	
  	MOM_do_template cycle_call
}
#=============================================================
proc MOM_bore_no_drag { } {
#=============================================================
global dwell_at_top
  	global surface_coordinate
  	global mom_cycle_delay_mode
  	global mom_cycle_delay
  	global mom_cycle_cam
  	global mom_cycle_feed_to
  	
  	set mom_cycle_cam "0"
   	
   	MOM_force once Q200 Q201 Q206 Q202 Q210 Q203 Q204 Q211 Q212 Q213 Q205 Q208 Q256
  	
  	set surface_coordinate "0.0"
	set dwell_at_top "0.0"
	if {![info exists mom_cycle_delay] || [$mom_cycle_delay_mode == "OFF"]} {
		set dwell_at_top 0.0
	} else {
		set dwell_at_top $mom_cycle_delay
	}
  	MOM_do_template cycle_203
  	MOM_suppress always N
  	MOM_do_template tnc_q200
  	MOM_do_template tnc_q201
  	MOM_do_template tnc_q206
  	MOM_do_template tnc_q202_1
  	MOM_do_template tnc_q210
  	MOM_do_template tnc_q203
 	MOM_do_template tnc_q204
 	MOM_do_template tnc_q212
	set mom_cycle_cam $mom_cycle_cam
  	MOM_do_template tnc_q213
  	MOM_do_template tnc_q205
  	MOM_do_template tnc_q211
  	MOM_do_template tnc_q208
	MOM_force once Q256
  	MOM_do_template tnc_q256
  	MOM_suppress off    N	

}
#=============================================================
proc  MOM_bore_no_drag_move { } {
#=============================================================
   	global mom_pos
   	global mom_prev_pos
   	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   	}	
  	MOM_do_template cycle_call
}
#=============================================================
proc MOM_bore_manual { } {
#=============================================================
global dwell_at_top
  	global surface_coordinate
  	global mom_cycle_delay_mode
  	global mom_cycle_delay
  	global mom_cycle_cam
  	global mom_cycle_feed_to
  	
  	set mom_cycle_cam "0"
   	
   	MOM_force once Q200 Q201 Q206 Q202 Q210 Q203 Q204 Q211 Q212 Q213 Q205 Q208 Q256
  	
  	set surface_coordinate "0.0"
  	set dwell_at_top "0.0"
  	if {![info exists mom_cycle_delay] || [$mom_cycle_delay_mode == "OFF"]} {
  		set dwell_at_top 0.0
  	} else {
		set 	dwell_at_top $mom_cycle_delay
    	}
  	MOM_do_template cycle_203
  	MOM_suppress always N
  	MOM_do_template tnc_q200
  	MOM_do_template tnc_q201
  	MOM_do_template tnc_q206
  	MOM_do_template tnc_q202_1
  	MOM_do_template tnc_q210
  	MOM_do_template tnc_q203
 	MOM_do_template tnc_q204
 	MOM_do_template tnc_q212
	set mom_cycle_cam $mom_cycle_cam
  	MOM_do_template tnc_q213
  	MOM_do_template tnc_q205
  	MOM_do_template tnc_q211
  	MOM_do_template tnc_q208
	MOM_force once Q256
  	MOM_do_template tnc_q256
  	MOM_suppress off    N	
}
#=============================================================
proc  MOM_bore_manual_move { } {
#=============================================================
   	global mom_pos
   	global mom_prev_pos
   	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   	}	
  	MOM_do_template cycle_call
}
#=============================================================
proc MOM_bore_dwell { } {
#=============================================================
global dwell_at_top
  	global surface_coordinate
  	global mom_cycle_delay_mode
  	global mom_cycle_delay
  	global mom_cycle_cam
  	global mom_cycle_feed_to
  	
  	set mom_cycle_cam "0"
   	
   	MOM_force once Q200 Q201 Q206 Q211 Q208 Q203 Q204 Q214 Q336 
  	
  	set surface_coordinate "0.0"
  	set dwell_at_top "0.0"
  	if {![info exists mom_cycle_delay] || [$mom_cycle_delay_mode == "OFF"]} {
  		set dwell_at_top 0.0
  	} else {
		set 	dwell_at_top $mom_cycle_delay
    	}
  	MOM_do_template cycle_202
  	MOM_suppress always N
  	MOM_do_template tnc_q200
  	MOM_do_template tnc_q201
  	MOM_do_template tnc_q206
  	#MOM_do_template tnc_q202_1
  	MOM_do_template tnc_q211
  	MOM_do_template tnc_q208
 	MOM_do_template tnc_q203
 	MOM_do_template tnc_q204
	set mom_cycle_cam $mom_cycle_cam
  	MOM_do_template tnc_q214
  	MOM_do_template tnc_q336
  	MOM_suppress off    N	
}
#=============================================================
proc  MOM_bore_dwell_move { } {
#=============================================================
   	global mom_pos
   	global mom_prev_pos
   	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   	}
  	MOM_do_template cycle_call
}
#=============================================================
proc MOM_bore_back { } {
#=============================================================
global dwell_at_top
  	global surface_coordinate
  	global mom_cycle_delay_mode
  	global mom_cycle_delay
  	global mom_cycle_cam
  	global mom_cycle_feed_to
  	
  	set mom_cycle_cam "0"
   	
   	MOM_force once Q200 Q249 Q250 Q251 Q252 Q253 Q254 Q255 Q203 Q204 Q214 Q336 
  	
  	set surface_coordinate "0.0"
  	set dwell_at_top "0.0"
  	if {![info exists mom_cycle_delay] || [$mom_cycle_delay_mode == "OFF"]} {
  		set dwell_at_top 0.0
  	} else {
		set 	dwell_at_top $mom_cycle_delay
    	}
  	MOM_do_template cycle_204
  	MOM_suppress always N
  	MOM_do_template tnc_q200
  	MOM_do_template tnc_q249
  	MOM_do_template tnc_q250
  	#MOM_do_template tnc_q202_1
  	MOM_do_template tnc_q251
  	MOM_do_template tnc_q252
 	MOM_do_template tnc_q253
 	MOM_do_template tnc_q254
 	MOM_do_template tnc_q255
	MOM_do_template tnc_q203
 	MOM_do_template tnc_q204
	set mom_cycle_cam $mom_cycle_cam
  	MOM_do_template tnc_q214
  	MOM_do_template tnc_q336
  	MOM_suppress off    N	
}
#=============================================================
proc  MOM_bore_back_move { } {
#=============================================================
   	global mom_pos
   	global mom_prev_pos
   	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   	}
  	MOM_do_template cycle_call
}
#=============================================================
proc MOM_bore_manual_dwell { } {
#=============================================================
global dwell_at_top
  	global surface_coordinate
  	global mom_cycle_delay_mode
  	global mom_cycle_delay
  	global mom_cycle_cam
  	global mom_cycle_feed_to
  	
  	set mom_cycle_cam "0"
   	
   	MOM_force once Q200 Q201 Q206 Q202 Q210 Q203 Q204 Q211 Q212 Q213 Q205 Q208 Q256
  	
  	set surface_coordinate "0.0"
  	set dwell_at_top "0.0"
  	if {![info exists mom_cycle_delay] || [$mom_cycle_delay_mode == "OFF"]} {
  		set dwell_at_top 0.0
  	} else {
		set 	dwell_at_top $mom_cycle_delay
    	}
  	MOM_do_template cycle_203
  	MOM_suppress always N
  	MOM_do_template tnc_q200
  	MOM_do_template tnc_q201
  	MOM_do_template tnc_q206
  	MOM_do_template tnc_q202_1
  	MOM_do_template tnc_q210
  	MOM_do_template tnc_q203
 	MOM_do_template tnc_q204
 	MOM_do_template tnc_q212
	set mom_cycle_cam $mom_cycle_cam
  	MOM_do_template tnc_q213
  	MOM_do_template tnc_q205
  	MOM_do_template tnc_q211
  	MOM_do_template tnc_q208
	MOM_force once Q256
  	MOM_do_template tnc_q256
  	MOM_suppress off    N	


}
#=============================================================
proc  MOM_bore_manual_dwell_move { } {
#=============================================================
   	global mom_pos
   	global mom_prev_pos
   	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_ac
   		MOM_do_template rapid_cycle_xy
   	}
  	MOM_do_template cycle_call
}

proc  MOM_drill_counter_sink {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a drill cycle.
#_______________________________________________________________________________
	MOM_drill
}

proc  MOM_drill_counter_sink_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a drill cycle.
#_______________________________________________________________________________
	MOM_drill_move
}

proc  MOM_drill_csink_dwell {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a drill dwell cycle.
#_______________________________________________________________________________
	MOM_drill_dwell
}

proc  MOM_drill_csink_dwell_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a drill dwell cycle.
#_______________________________________________________________________________
	MOM_drill_dwell_move
}

proc  MOM_load_tool {} {
#_______________________________________________________________________________
# This procedure is executed for the LOAD/TOOL command.  All output for the LOAD
# command will take place with the initial move.
#_______________________________________________________________________________
}

proc  MOM_sequence_number {} {
#_______________________________________________________________________________
# This procedure is executed for the Sequence Number command.
#_______________________________________________________________________________
	global mom_sequence_mode
	global mom_sequence_number
	global mom_sequence_increment
	global mom_sequence_frequency
	global mom_sequence_text
	if {[isset mom_sequence_mode] == "y"} {
		if {$mom_sequence_mode == "OFF"} {
			MOM_set_seq_off
		} elseif {$mom_sequence_mode == "ON"} {
			MOM_set_seq_on
		} elseif {$mom_sequence_mode == "AUTO"} {
			MOM_output_literal "SEQNO/AUTO "
		} else {
			MOM_output_literal "error: mom_sequence_mode unknown"
		}
	} else {
		MOM_reset_sequence $mom_sequence_number $mom_sequence_increment $mom_sequence_frequency
	}
}

proc  MOM_set_modes {} {
#_______________________________________________________________________________
# This procedure is executed for the Set Mode command.
#_______________________________________________________________________________
	global mom_output_mode
	global mom_arc_mode
	global mom_feed_rate_output_mode
	global mom_parallel_to_axis
	global mom_machine_mode
	global mom_modes_text
	global g_output_mode

# "ABSOLUTE", "INCREMENTAL"
	if {$mom_output_mode == "ABSOLUTE"} {
		set g_output_mode "G90"
		MOM_incremental OFF X Y Z Axis_4th Axis_5th
	} elseif {$mom_output_mode == "INCREMENTAL"} {
		set g_output_mode "G91"
		MOM_incremental ON X Y Z Axis_4th Axis_5th
	}
# "LINEAR", "CIRCULAR"
	if {$mom_arc_mode == "LINEAR"} {
	} elseif {$mom_arc_mode == "CIRCULAR"} {
	}
# "OFF", "IPM", "IPR", "MMPM", "MMPR", "INVERSE"
	if {$mom_feed_rate_output_mode == "FPM"} {
	} elseif {$mom_feed_rate_output_mode == "FPR"} {
	} elseif {$mom_feed_rate_output_mode == "INVERSE"} {
	} elseif {$mom_feed_rate_output_mode == "OFF"} {
	}
# "ZAXIS", "WAXIS", "UAXIS"
	if {$mom_parallel_to_axis == "ZAXIS"} {
	} elseif {$mom_parallel_to_axis == "WAXIS"} {
	} elseif {$mom_parallel_to_axis == "UAXIS"} {
	}
# "MILL", "TURN", "PUNCH", "LASER", "TORCH", "WIRE"
	if {$mom_machine_mode == "MILL"} {
	} elseif {$mom_machine_mode == "TURN"} {
	} elseif {$mom_machine_mode == "PUNCH"} {
	} elseif {$mom_machine_mode == "LASER"} {
	} elseif {$mom_machine_mode == "TORCH"} {
	} elseif {$mom_machine_mode == "WIRE"} {
	}
}

#=============================================================
proc MOM_cutcom_on { } {
#=============================================================
	global mom_cutcom_mode
	global cutcom_mode 
	global mom_cutcom_status
	global cutcom_on
	
	set cutcom_mode "R"
   	if {$mom_cutcom_mode == "OFF"} {
      		set cutcom_mode "0"
   	} elseif {$mom_cutcom_mode == "RIGHT"} {
      		set cutcom_mode "R"
   	} elseif {$mom_cutcom_mode == "LEFT"} {
      		set cutcom_mode "L"
   	} elseif {$mom_cutcom_mode == "ON"} {
      		set cutcom_mode "R"
   	}
   	if {$mom_cutcom_status == "OFF"} {
   		set 	cutcom_mode "0"
   	}
   	set cutcom_on 1
}
#=============================================================
proc MOM_cutcom_off { } {
#=============================================================
	global cutcom_mode
	set cutcom_mode "0"
	set cutcom_on 0
	MOM_force once R
}

#=============================================================
proc MOM_spindle_rpm { } {
#=============================================================
	global mom_tool_number
	global mom_spindle_speed
	global line
	global mom_tool_name mom_tool_diameter

   	set line [format "TOOL CALL %d Z S%.0f ; TOOL NAME = %s RADIUS = %.2f" $mom_tool_number $mom_spindle_speed $mom_tool_name [expr $mom_tool_diameter/2]]
   	MOM_output_literal "$line"	
   	MOM_do_template spindle_rpm 	
}

proc  MOM_spindle_off {} {
#_______________________________________________________________________________
# This procedure is executed when the Spindle command is deactivated.
#_______________________________________________________________________________
         MOM_do_template spindle_off
}

proc  MOM_coolant_on {} {
#_______________________________________________________________________________
# This procedure is executed when the Coolant command is activated.
#_______________________________________________________________________________
	global mom_coolant_text
#MOM_do_template coolant_on
}

proc  MOM_coolant_off {} {
#_______________________________________________________________________________
# This procedure is executed when the Coolant command is deactivated.
#_______________________________________________________________________________
	MOM_do_template coolant_off
}

proc  MOM_opstop {} {
#_______________________________________________________________________________
# This procedure is executed when the Optional Stop command is activated.
#_______________________________________________________________________________
	MOM_do_template opstop
}

proc  MOM_stop {} {
#_______________________________________________________________________________
# This procedure is executed when the Program Stop command is activated.
#_______________________________________________________________________________
	MOM_do_template stop
}

proc  MOM_opskip_on {} {
#_______________________________________________________________________________
# This procedure is executed when the Optional skip command is activated.
#_______________________________________________________________________________
	MOM_set_line_leader always "/"
}

proc  MOM_opskip_off {} {
#_______________________________________________________________________________
# This procedure is executed when the Optional skip command is activated.
#_______________________________________________________________________________
	MOM_set_line_leader off "/"
}

proc  MOM_delay {} {
#_______________________________________________________________________________
# This procedure is executed when the Delay command is activated.
#_______________________________________________________________________________
	MOM_do_template delay
}

proc  MOM_auxfun {} {
#_______________________________________________________________________________
# This procedure is executed when the Auxiliary command is activated.
#_______________________________________________________________________________
	MOM_do_template auxiliary
}

proc  MOM_prefun {} {
#_______________________________________________________________________________
# This procedure is executed when the Preparatory function command is activated.
#_______________________________________________________________________________
	MOM_do_template preparatory
}

proc  MOM_length_compensation {} {
#_______________________________________________________________________________
# This procedure is executed when the Tool Length Compensation command is activated.
#_______________________________________________________________________________
}

proc  MOM_tool_preselect {} {
#_______________________________________________________________________________
# This procedure is executed when the Tool Preselect command is activated.
#_______________________________________________________________________________
}

proc ZERO_code {} {

	global mom_record_fields
	global mom_field_count
	global zero_pos
	global zero_g_code
	global prev_motion_g

 # if {[DATATYPE mom_record_fields(1)] == "int"} {
 #     set mom_record_fields(1) [expr 53 + $mom_record_fields(1)]
 #     MOM_output_literal "G$mom_record_fields(1)"
 # } else {
 #     switch $mom_record_fields(1) {
 #       "CHECK" { set zero_g_code 27 }
  #      "ON"    { set zero_g_code 28 }
  #      "OFF"   { set zero_g_code 29 }
 #       "THRU"  { set zero_g_code 30 }
 #     }
 # }
	MOM_output_literal "CYCL DEF 7.0 DATUM SHIFT"
	MOM_output_literal "CYCL DEF 7.1 #$mom_record_fields(1)"
}

proc DATATYPE { v1 } {
	upvar $v1 v2
	set signed_flag 0
	set int_flag 0
	set char_flag 0
	set real_flag 0
	set str_len [string length $v2]

	for {set i 0} {$i < $str_len} {incr i} {
		set char [string index $v2 $i]
		if {$char == "-" || $char == "+"} {
			set signed_flag 1
		} else {
			switch -regexp $char {
				[0-9] { set int_flag 1 }
				[a-z] { set char_flag 1
					set int_flag 0
					set real_flag 0
					set signed_flag 0
					break
				}
				[A-Z] { set char_flag 1
					set int_flag 0
					set real_flag 0
					set signed_flag 0
					break
				}
				"%"   { set char_flag 1
					set int_flag 0
					set real_flag 0
					set signed_flag 0
					break
				}
				"."   { set real_flag 1 }
				default { set char_flag 1
					set int_flag 0
					set real_flag 0
					set signed_flag 0
					break
				}
			}
		}
	}
	if {$char_flag == 1} {
      		return "string"
	} elseif {$real_flag == 1} {
		if {$signed_flag == 1} {
			return "signed_real"
		} else {
			return "real"
		}
	} elseif {$int_flag == 1} {
      		if {$signed_flag == 1} {
          		return "signed_int"
      		} else {
          		return "int"
      		}
  	} elseif {$signed_flag == 1} {
      		return "sign"
  	}
}

proc SET_G_PLANE {} {
	global mom_out_angle_pos
	global g_plane
	global MCSROT_flag

	if {$MCSROT_flag == 1} {
		set g_plane ""
		return 
	}
	set axis_4th [format "%-.3f" $mom_out_angle_pos(0)]
	if {abs($axis_4th) == 180.0} {
		set g_plane 18
		MOM_force once plane_g
		MOM_suppress always J
	} elseif {abs($axis_4th) == 0.0 || abs($axis_4th) == 360.0} {
		set g_plane 17
		MOM_force once plane_g
		MOM_suppress always K
	} else {
		set g_plane ""
	}
}

proc GET_PART_NAME { v1 } {

	global plat_unix
	global plat_wnt

	set plat_unix 0
	set plat_wnt 0
	set mom_part_name $v1

# Determine Platform

# Unix
	set plat_unix [regexp "/" $mom_part_name]
# Windows NT
	set plat_wnt [regexp {\\} $mom_part_name]

	if {$plat_unix == 1} {
      		set partno_list [split $mom_part_name "/"]
  	} elseif {$plat_wnt == 1} {
      		set partno_list [split $mom_part_name "\\"]
  	} else {
      		set partno_list {$mom_part_name}
  	}
# Determine the part name
	set num [llength $partno_list]
	set num [expr $num -1]
	set part_name [lindex $partno_list $num]
	set num [string length $part_name]
	set num [expr $num - 5]
	set part_name [string range $part_name 0 $num]
	set num [string length $part_name]

# Convert the to upper case

	set part_name [string toupper $part_name]
	return $part_name
}

proc  MOM_text {} {
#_______________________________________________________________________________
# This procedure is executed when the Text command is activated.
#_______________________________________________________________________________
	global mom_user_defined_text
	global mom_record_fields
	global mom_record_text
	global MCSROT_mode
	global MCSROT_flag
	global is_MCSROT_matrix_set
	global first_move_after_MCSROT
	global prog_number

	switch $mom_record_fields(0) {
	"INSERT"
	{
		MOM_output_literal $mom_record_text
	}
	"ZERO"
	{
		ZERO_code
	}
	"MCSROT"
	{
		set MCSROT_mode $mom_record_fields(1)
		if {$mom_record_fields(1) == "OFF"} {
			set MCSROT_flag 0
			set is_MCSROT_matrix_set 0
			set first_move_after_MCSROT 0
			MOM_suppress off K
			MOM_do_template cancel_rotate_mcs
		} else {
			set MCSROT_flag 1
		}
	}
	"PARTNO"
	{
		set mom_record_text [string trim $mom_record_text " "]
		set temp [split $mom_record_text " "]
		set prog_number [lindex $temp 0]
		PROGRAM_START
	}
	default
		{ MOM_output_literal "$mom_user_defined_text" }
	}
}

#--------------------------------------------------------------------------------------------------- 
# Procedure for Kevins IK Library and Coordinate system rotation 
#---------------------------------------------------------------------------------------------------

# Setting default variables
set first_move_after_MCSROT	0
set MCSROT_flag			0
set is_MCSROT_matrix_set	0
set first_motion_flag			0

#_______________________________________________________________________________
# This procedure is executed before the motion event is activated.
#_______________________________________________________________________________

proc MOM_before_motion {} {

	global mom_mcs_goto
	global mom_tool_axis
	global mom_alt_pos
	global mom_pos
	global mom_out_angle_pos
	global mom_motion_type
	global mom_current_motion
	global mom_pos_a1
	global mom_alt_pos_a1
  
	FEEDRATE_SET
	SET_MCS_MATRIX
	
	if {$mom_current_motion == "circular_move"} {
		CONVERT_ARC_DATA
	}
	if {$mom_motion_type != "CYCLE"} {
      		CONVERT_POINT mom_mcs_goto \
			mom_tool_axis \
			mom_pos \
			mom_alt_pos \
			mom_out_angle_pos
	} else {
		CONVERT_CYCLES_DATA
	}
}

proc MOM_origin {} {
	global mom_origin
	global mom_origin_x mom_origin_y mom_origin_z

	set mom_origin_x $mom_origin(0)
	set mom_origin_y $mom_origin(1)
	set mom_origin_z $mom_origin(2)
}

proc SET_ROTARY_ANGLE {v1} {
	global prev_c_value
	global prev_theta_value
######  global full_rev     ;# not referenced (whb)

	set angle $v1
	set diff [expr $angle - $prev_theta_value]
 
	if {[EQ_is_zero $diff]} {
		set direction 0
	} elseif {abs($diff) < 180} {
		if {$angle > $prev_theta_value} {
			set direction -1
		} else {
			set direction 1
		}
	} else {  
		if {$angle > $prev_theta_value} {
			set direction 1
		} else {
			set direction -1
		}
	}

	if {$direction == 1} {
		if {$angle >= $prev_c_value} {
			while {$angle > $prev_c_value} {
				set angle [expr $angle - 360]
			}
		} else {
			while {$angle < $prev_c_value} {
				set angle [expr $angle + 360]
			}
			set angle [expr $angle - 360.0]
		}
	} elseif {$direction == -1} {
		if {$angle <= $prev_c_value} {
			while {$angle < $prev_c_value} {
				set angle [expr $angle + 360]
			}
		} else {
			while {$angle > $prev_c_value} {
				set angle [expr $angle - 360]
			}
			set angle [expr $angle + 360.0]
		}
	} else {
		set angle $prev_c_value
	}
	return $angle
}
  
proc OUTPUT_MCS_ROTATION {} {
	global MCSROT_axis_A 
	global MCSROT_axis_B 
	global MCSROT_axis_C
	global is_MCSROT_matrix_set
	global trans_axis trans_angle

	if {$is_MCSROT_matrix_set} {
		for {set i 0} {$i < 3} {incr i} {
			switch $trans_axis($i) {
				"AAXIS" {set MCSROT_axis_A $trans_angle($i)}
				"BAXIS" {set MCSROT_axis_B $trans_angle($i)}
				"CAXIS" {set MCSROT_axis_C $trans_angle($i)}
			}
		}
		MOM_do_template rotate_mcs
	}
}

proc SET_MCS_MATRIX {} {
	global mom_operation_type
	global mom_motion_type
	global mom_tool_axis
	global mom_prev_tool_axis
	global is_MCSROT_matrix_set
	global MCSROT_flag
	global trans_axis 
	global trans_angle
	global first_motion_flag
	global first_move_after_MCSROT

# Check if the MCSROT is programmed, return if not

	if {$MCSROT_flag == 0} { return }

# Check if the initial_move or first_move have been triggered, if not return

	if {$first_motion_flag == 0} { return }

# Check if the operation type is compatible for MCS rotation

	if {$mom_operation_type == "VAR_CONTOUR"} {
		MOM_output_to_listing_device \
		"ERROR: ***** INVALID OPERATION TYPE FOR MCS ROTATION *****"
		MOM_output_to_listing_device \
		"OPERATION - $mom_path_name"
		MOM_output_to_listing_device \
		"MESSAGE: Variable Axis Contouring type of operations are invalid"
		MOM_output_to_listing_device \
		"         when in Coordinate System Rotation mode"
		return
	}

# Check if the rotation matrix has been set up, if yes do not do anything.

	if {$is_MCSROT_matrix_set == 0} {
		SET_MCS_ROTATION_ANGLES
		set deg2rad [expr asin(1.0)/90.0]
		MOM_rotate_mach_csys $trans_axis(0) $trans_angle(0) \
		$trans_axis(1) $trans_angle(1) \
		$trans_axis(2) $trans_angle(2)]
		set first_move_after_MCSROT 1
		MOM_suppress always K
		set is_MCSROT_matrix_set 1
		OUTPUT_MCS_ROTATION
	}
}

#----------------------------------------------------------------------------
#
# Function LOAD_5AXIS_KINEMATICS copies the value of the kinematic variables
# into some of the variables that UF_MOM_ask_string in User Function can get.
# This was the limitation in V15. 
# In V16, however the call to this Function may not be necessary as User 
# Function call are available to get the tcl array values
#
#----------------------------------------------------------------------------

proc LOAD_5AXIS_KINEMATICS {} {

# Kinematics for the DECKEL MAHO DMC 50 5 Axis machine
	global mom_kin_x_axis_limit
	global mom_kin_y_axis_limit
	global mom_kin_z_axis_limit
	global mom_kin_machine_resolution
	global mom_kin_machine_type
	global mom_kin_4th_axis_plane
	global mom_kin_4th_axis_vector
	global mom_kin_4th_axis_angles
	global mom_kin_4th_axis_min_incr 
	global mom_kin_4th_axis_min_limit 
	global mom_kin_4th_axis_max_limit 
	global mom_kin_4th_axis_direction 
	global mom_kin_4th_axis_rotation 
	global mom_kin_4th_axis_zero 
	global mom_kin_4th_axis_center_offset 
	global mom_kin_pivot_gauge_offset 
	global mom_kin_gauge_to_pivot
	global mom_kin_5th_axis_plane 
	global mom_kin_5th_axis_vector 
	global mom_kin_5th_axis_angles 
	global mom_kin_5th_axis_min_incr 
	global mom_kin_5th_axis_min_limit 
	global mom_kin_5th_axis_max_limit 
	global mom_kin_5th_axis_direction 
	global mom_kin_5th_axis_rotation 
	global mom_kin_5th_axis_inclination_to_4th 
	global mom_kin_5th_axis_zero 
	global mom_kin_5th_axis_center_offset
	global mom_kin_linearization_tol 
	global mom_kin_linearization_flag 
	global mom_kin_independent_head 
	global mom_kin_nurbs_output_type 
	global mom_kin_output_unit 
	global mom_kin_4th_axis_center_offset_x
	global mom_kin_4th_axis_center_offset_y
	global mom_kin_4th_axis_center_offset_z
	global mom_kin_4th_axis_angles_a1
	global mom_kin_4th_axis_angles_a2
	global mom_kin_4th_axis_vector_i
	global mom_kin_4th_axis_vector_j
	global mom_kin_4th_axis_vector_k
	global mom_kin_track_center_of_ball
	global mom_kin_read_ahead_next_motion
 	global leader_4th leader_5th
	global tcl_platform

  	if {$tcl_platform(os) == "HP-UX"} {
		set plat "hpp"
		set suffix "sl"
	} elseif {$tcl_platform(os) == "windows" || \
		$tcl_platform(os) == "Windows NT"} {
		set plat "wnt"
		set suffix "dll"
	} else {
		set plat "sun"
		set suffix "so"
	}
	set uf_library "ugpadvkins.$suffix"

	set mom_kin_read_ahead_next_motion	TRUE
	set mom_kin_machine_resolution		0.001
	set mom_kin_machine_type			5_axis_dual_table
	set mom_kin_track_center_of_ball		FALSE
	set mom_kin_4th_axis_plane			NONE
	set mom_kin_4th_axis_vector(0)		0.000
	set mom_kin_4th_axis_vector(1)		0.000
	set mom_kin_4th_axis_vector(2)		1.000
	set mom_kin_4th_axis_min_incr		0.001
	set mom_kin_4th_axis_min_limit		-5400.000
	set mom_kin_4th_axis_max_limit		5400.000
	set mom_kin_4th_axis_direction		SIGN_DETERMINES_DIRECTION
	set mom_kin_4th_axis_rotation			standard
	set mom_kin_4th_axis_zero			0
	set mom_kin_4th_axis_center_offset(0)	0.0
	set mom_kin_4th_axis_center_offset(1)	0.0
	set mom_kin_4th_axis_center_offset(2)	0.0
#	set mom_kin_4th_axis_center_offset(1)	-310.000
#	set mom_kin_4th_axis_center_offset(2)	155.036
	set mom_kin_pivot_gauge_offset		0.0
	set mom_kin_gauge_to_pivot(0)		0.0
	set mom_kin_gauge_to_pivot(1)		0.0
	set mom_kin_gauge_to_pivot(2)		0.0
	
	set mom_kin_5th_axis_plane			NONE
	set mom_kin_5th_axis_vector(0)		0.000
	set mom_kin_5th_axis_vector(1)		1.000
	set mom_kin_5th_axis_vector(2)		1.000
	set mom_kin_5th_axis_min_incr		0.001
	set mom_kin_5th_axis_min_limit		0.000
	set mom_kin_5th_axis_max_limit		180.000
	set mom_kin_5th_axis_direction		SIGN_DETERMINES_DIRECTION
	set mom_kin_5th_axis_rotation			standard
	set mom_kin_5th_axis_inclination_to_4th	45.000
	set mom_kin_5th_axis_zero			0

	set mom_kin_5th_axis_center_offset(0)	0.0
#	set mom_kin_5th_axis_center_offset(1)	319.950
	set mom_kin_5th_axis_center_offset(2)	-155.036
	set mom_kin_5th_axis_center_offset(1)	-310.072
#	set mom_kin_5th_axis_center_offset(2)	-154.975

	set mom_kin_linearization_tol			0.001
	set mom_kin_linearization_flag			TRUE ; # T ON, F OFF
	set mom_kin_independent_head		NONE
	set mom_kin_nurbs_output_type		HEIDENHAIN_POLY

	set leader_4th "C"
	set leader_5th "B"

	global mom_kin_5th_axis_center_offset_x
	global mom_kin_5th_axis_center_offset_y
	global mom_kin_5th_axis_center_offset_z
	global mom_kin_5th_axis_angles_a1
	global mom_kin_5th_axis_angles_a2
	global mom_kin_5th_axis_vector_i
	global mom_kin_5th_axis_vector_j
	global mom_kin_5th_axis_vector_k
	global mom_kin_gauge_to_pivot_x
	global mom_kin_gauge_to_pivot_y
	global mom_kin_gauge_to_pivot_z
	global mom_kin_4th_axis_limit_action
	global mom_kin_5th_axis_limit_action
  
	set mom_kin_4th_axis_limit_action		"Retract / Re-Engage"
	set mom_kin_5th_axis_limit_action		"Retract / Re-Engage"
  	set mom_kin_4th_axis_center_offset_x $mom_kin_4th_axis_center_offset(0)
	set mom_kin_4th_axis_center_offset_y $mom_kin_4th_axis_center_offset(1) 
	set mom_kin_4th_axis_center_offset_z $mom_kin_4th_axis_center_offset(2)
	set mom_kin_5th_axis_center_offset_x $mom_kin_5th_axis_center_offset(0)
	set mom_kin_5th_axis_center_offset_y $mom_kin_5th_axis_center_offset(1) 
	set mom_kin_5th_axis_center_offset_z $mom_kin_5th_axis_center_offset(2)
	set mom_kin_4th_axis_vector_i $mom_kin_4th_axis_vector(0)
	set mom_kin_4th_axis_vector_j $mom_kin_4th_axis_vector(1)
	set mom_kin_4th_axis_vector_k $mom_kin_4th_axis_vector(2)
	set mom_kin_5th_axis_vector_i $mom_kin_5th_axis_vector(0)
	set mom_kin_5th_axis_vector_j $mom_kin_5th_axis_vector(1)
	set mom_kin_5th_axis_vector_k $mom_kin_5th_axis_vector(2)

	switch $mom_kin_4th_axis_plane {
		"XY" -
		"YX"
		{
			set mom_kin_4th_axis_vector_i		0.0
			set mom_kin_4th_axis_vector_j		0.0
			set mom_kin_4th_axis_vector_k		1.0
         		}
    		"YZ" -
    		"ZY"
         		{
			set mom_kin_4th_axis_vector_i		1.0
			set mom_kin_4th_axis_vector_j		0.0
			set mom_kin_4th_axis_vector_k		0.0
		}
		"ZX" -
		"XZ"
		{
			set mom_kin_4th_axis_vector_i		0.0
			set mom_kin_4th_axis_vector_j		1.0
			set mom_kin_4th_axis_vector_k		0.0
		}
		default
		{
			if {[info exists mom_kin_4th_axis_vector]} {
				set mom_kin_4th_axis_vector_i $mom_kin_4th_axis_vector(0)
				set mom_kin_4th_axis_vector_j $mom_kin_4th_axis_vector(1)
				set mom_kin_4th_axis_vector_k $mom_kin_4th_axis_vector(2)
			} elseif {[info exists mom_kin_4th_axis_angles]} {
				set mom_kin_4th_axis_angles_a1 $mom_kin_4th_axis_angles(0)
				set mom_kin_4th_axis_angles_a2 $mom_kin_4th_axis_angles(1)
			} else {
				set mom_kin_4th_axis_vector_i		0.0
				set mom_kin_4th_axis_vector_j		0.0
				set mom_kin_4th_axis_vector_k		0.0
				set mom_kin_4th_axis_angles_a1		0.0
				set mom_kin_4th_axis_angles_a2		0.0
               			MOM_output_to_listing_device \
                   				"ERROR: Machine's 4th axis Kinematics are INCONSISTANT"
           		}
		}
	}

	switch $mom_kin_5th_axis_plane {
		"XY" -
		"YX"
		{
			set mom_kin_5th_axis_vector_i		0.0
			set mom_kin_5th_axis_vector_j		0.0
			set mom_kin_5th_axis_vector_k		1.0
		}
    		"YZ" -
    		"ZY"
		{
			set mom_kin_5th_axis_vector_i		1.0
			set mom_kin_5th_axis_vector_j		0.0
			set mom_kin_5th_axis_vector_k		0.0
		}
		"ZX" -
		"XZ"
		{
			set mom_kin_5th_axis_vector_i		0.0
			set mom_kin_5th_axis_vector_j		1.0
			set mom_kin_5th_axis_vector_k		0.0
		}
		default
		{
			if {[info exists mom_kin_5th_axis_vector]} {
				set mom_kin_5th_axis_vector_i $mom_kin_5th_axis_vector(0)
				set mom_kin_5th_axis_vector_j $mom_kin_5th_axis_vector(1)
				set mom_kin_5th_axis_vector_k $mom_kin_5th_axis_vector(2)
			} elseif {[info exists mom_kin_5th_axis_angles]} {
				set mom_kin_5th_axis_angles_a1 $mom_kin_5th_axis_angles(0)
				set mom_kin_5th_axis_angles_a2 $mom_kin_5th_axis_angles(1)
			} else {
				set mom_kin_5th_axis_vector_i		0.0
				set mom_kin_5th_axis_vector_j		0.0
				set mom_kin_5th_axis_vector_k		0.0
				set mom_kin_5th_axis_angles_a1		0.0
				set mom_kin_5th_axis_angles_a2		0.0
				MOM_output_to_listing_device \
					"ERROR: Machine's 5th axis Kinematics are INCONSISTANT"
			}
		}
	}
	MOM_run_user_function [MOM_ask_env_var UGII_CAM_AUXILIARY_DIR]$uf_library ufusr
	MOM_load_kinematics
}

proc CONVERT_POINT  {
		mcs_goto \
		tool_axis \
		pos \
		alt_pos \
		out_angle_pos \
		} \
{
	upvar $mcs_goto mom_mcs_goto
	upvar $tool_axis mom_tool_axis
	upvar $pos mom_pos
	upvar $alt_pos mom_alt_pos
	upvar $out_angle_pos mom_out_angle_pos

	global mom_pos_x mom_pos_y mom_pos_z
	global mom_pos_a1 mom_pos_a2
	global mom_mcs_goto_x mom_mcs_goto_y mom_mcs_goto_z
	global mom_tool_axis_i mom_tool_axis_j mom_tool_axis_k
	global mom_alt_pos_x mom_alt_pos_y mom_alt_pos_z
	global mom_alt_pos_a1 mom_alt_pos_a2
	global mom_kin_5th_axis_min_limit
	global mom_kin_5th_axis_max_limit
	global first_move_after_MCSROT
	global MCSROT_flag
    	global c_value
	global theta_value
	global prev_c_value
	global prev_theta_value mom_alt_pos mom_pos
	global mom_rotary_delta_4th mom_rotary_delta_5th

	set mom_mcs_goto_x $mom_mcs_goto(0)
	set mom_mcs_goto_y $mom_mcs_goto(1)
	set mom_mcs_goto_z $mom_mcs_goto(2)

	set mom_tool_axis_i $mom_tool_axis(0)
	set mom_tool_axis_j $mom_tool_axis(1)
	set mom_tool_axis_k $mom_tool_axis(2)
#  
	set mom_pos(3) [format %.7f $mom_pos(3)]
	if {$mom_pos(3) == 45.0000000} {
		MOM_output_literal "; WARNING! The limitation of axis rotation"
		set mom_tool_axis_i  [expr $mom_tool_axis_i+0.001]
		set mom_tool_axis_j  [expr $mom_tool_axis_j+0.001]
		set mom_tool_axis_k [expr $mom_tool_axis_k+0.001]
	}
#   	
	if {$first_move_after_MCSROT == 1 && $MCSROT_flag == 1} {
		MOM_map_to_rotated_csys
	} else {
		MOM_convert_point
	}
	
	while {$mom_pos_a1 < 0.0} {set mom_pos_a1 [expr $mom_pos_a1+360.0]}
	while {$mom_pos_a1 >= 360.0} {set mom_pos_a1 [expr $mom_pos_a1-360.0]}
	while {$mom_pos_a2 < 0.0} {set mom_pos_a2 [expr $mom_pos_a2+360.0]}
	while {$mom_pos_a2 >= 360.0} {set mom_pos_a2 [expr $mom_pos_a2-360.0]}
	while {$mom_alt_pos_a1 < 0.0} {set mom_alt_pos_a1 [expr $mom_alt_pos_a1+360.0]}
	while {$mom_alt_pos_a1 >= 360.0} {set mom_alt_pos_a1 [expr $mom_alt_pos_a1-360.0]}
	while {$mom_alt_pos_a2 < 0.0} {set mom_alt_pos_a2 [expr $mom_alt_pos_a2+360.0]}
	while {$mom_alt_pos_a2 >= 360.0} {set mom_alt_pos_a2 [expr $mom_alt_pos_a2-360.0]}

	set rot_5th_axis [format "%-.7f" $mom_pos_a2]

	if {[EQ_is_lt $rot_5th_axis $mom_kin_5th_axis_min_limit] || \
		[EQ_is_gt $rot_5th_axis $mom_kin_5th_axis_max_limit] } {

		set mom_pos(0) $mom_alt_pos_x
		set mom_pos(1) $mom_alt_pos_y
		set mom_pos(2) $mom_alt_pos_z
		set mom_pos(3) $mom_alt_pos_a1
		set mom_pos(4) $mom_alt_pos_a2
		set mom_alt_pos(0) $mom_pos_x
		set mom_alt_pos(1) $mom_pos_y
		set mom_alt_pos(2) $mom_pos_z
		set mom_alt_pos(3) $mom_pos_a1
		set mom_alt_pos(4) $mom_pos_a2
		set theta_value $mom_alt_pos_a1
		
		if {[info exist prev_theta_value]} {
			set c_value [SET_ROTARY_ANGLE $theta_value]
		} else {
			set c_value $theta_value
		}
		
			set mom_out_angle_pos(0) $c_value
			set mom_out_angle_pos(1) $mom_alt_pos_a2
		set prev_theta_value $theta_value 
		set prev_c_value $c_value
	} else {
		set mom_pos(0) $mom_pos_x
		set mom_pos(1) $mom_pos_y
		set mom_pos(2) $mom_pos_z
		set mom_pos(3) $mom_pos_a1
		set mom_pos(4) $mom_pos_a2
		set mom_alt_pos(0) $mom_alt_pos_x
		set mom_alt_pos(1) $mom_alt_pos_y
		set mom_alt_pos(2) $mom_alt_pos_z
		set mom_alt_pos(3) $mom_alt_pos_a1
		set mom_alt_pos(4) $mom_alt_pos_a2
		set theta_value $mom_pos_a1
		if {[info exist prev_theta_value]} {
			set c_value [SET_ROTARY_ANGLE $theta_value]
		} else {
			set c_value $theta_value
		}
		set mom_out_angle_pos(0) $c_value
		set mom_out_angle_pos(1) $mom_pos_a2
		set prev_theta_value $theta_value
		set prev_c_value $c_value
	}
}

proc CONVERT_CYCLES_DATA {} {

	global mom_mcs_goto
	global mom_tool_axis
	global mom_out_angle_pos
	global mom_cycle_rapid_to
	global mom_cycle_retract_to
	global mom_cycle_feed_to
	global mom_pos
	global mom_cycle_rapid_to_pos
	global mom_cycle_retract_to_pos
	global mom_cycle_feed_to_pos

	set dummy_alt_pos(0) 0.0
	set dummy_alt_pos(1) 0.0
	set dummy_alt_pos(2) 0.0
	set dummy_alt_pos(3) 0.0
	set dummy_alt_pos(4) 0.0

	CONVERT_POINT mom_mcs_goto \
		mom_tool_axis \
		mom_pos \
		mom_alt_pos \
		mom_out_angle_pos

	set retract_to_point(0) [expr $mom_mcs_goto(0) + \
		$mom_cycle_retract_to * $mom_tool_axis(0)]
	set retract_to_point(1) [expr $mom_mcs_goto(1) + \
		$mom_cycle_retract_to * $mom_tool_axis(1)]
	set retract_to_point(2) [expr $mom_mcs_goto(2) + \
		$mom_cycle_retract_to * $mom_tool_axis(2)]
  
	CONVERT_POINT retract_to_point \
		mom_tool_axis \
		mom_cycle_retract_to_pos \
		dummy_alt_pos \
		mom_out_angle_pos

	set rapid_to_point(0) [expr $mom_mcs_goto(0) + \
		$mom_cycle_rapid_to * $mom_tool_axis(0)]
	set rapid_to_point(1) [expr $mom_mcs_goto(1) + \
		$mom_cycle_rapid_to * $mom_tool_axis(1)]
	set rapid_to_point(2) [expr $mom_mcs_goto(2) + \
		$mom_cycle_rapid_to * $mom_tool_axis(2)]

	CONVERT_POINT rapid_to_point \
		mom_tool_axis \
		mom_cycle_rapid_to_pos \
		dummy_alt_pos \
		mom_out_angle_pos
 
	set feed_to_point(0) [expr $mom_mcs_goto(0) + \
		$mom_cycle_feed_to * $mom_tool_axis(0)]
	set feed_to_point(1) [expr $mom_mcs_goto(1) + \
		$mom_cycle_feed_to * $mom_tool_axis(1)]
	set feed_to_point(2) [expr $mom_mcs_goto(2) + \
		$mom_cycle_feed_to * $mom_tool_axis(2)]
	
	CONVERT_POINT feed_to_point \
		mom_tool_axis \
		mom_cycle_feed_to_pos \
		dummy_alt_pos \
		mom_out_angle_pos
}

proc CONVERT_ARC_DATA {} {

	global mom_arc_center
	global mom_prev_mcs_goto
	global mom_tool_axis
	global mom_pos_arc_center
	global mom_prev_pos
	global mom_prev_alt_pos

	set dummy_out_angle_pos(0) 0.0
	set dummy_out_angle_pos(1) 0.0
	set dummy_alt_pos(0) 0.0
	set dummy_alt_pos(1) 0.0
	set dummy_alt_pos(2) 0.0
	set dummy_alt_pos(3) 0.0
	set dummy_alt_pos(4) 0.0

	CONVERT_POINT mom_arc_center \
		mom_tool_axis \
		mom_pos_arc_center \
		dummy_alt_pos \
		dummy_out_angle_pos
	
	CONVERT_POINT mom_prev_mcs_goto \
		mom_tool_axis \
		mom_prev_pos \
		mom_prev_alt_pos \
		dummy_out_angle_pos
}

proc SET_MCS_ROTATION_ANGLES {} {
	global mom_record_fields
	global mom_field_count
	global mom_tool_axis
	global mom_origin
	global mom_translate

	global mom_translate_x mom_translate_y mom_translate_z
	global mom_origin_x mom_origin_y mom_origin_z
	global MCSROT_flag
	global first_move_after_MCSROT
	global trans_axis 
	global trans_angle
	global MCSROT_mode

	set MCSROT_flag 1

	switch $MCSROT_mode {
	"AUTO"
	{
		set I $mom_tool_axis(0)
		set J $mom_tool_axis(1)
		set K $mom_tool_axis(2)
		set rad2deg [expr 90.0/asin(1.0)]
		set reslt_JK [expr sqrt(pow($J,2)+pow($K,2))]
		set trans_axis(0) "AAXIS"
		set trans_axis(1) "BAXIS"
		set trans_axis(2) "0"
		set trans_angle(0) [expr -1.0*atan2($J,$K)*$rad2deg]
		set trans_angle(1) [expr atan2($I,$reslt_JK)*$rad2deg]
		set trans_angle(2) 0.0
	}
	default
	{ 
		if {$mom_field_count > 7 || \
		$mom_field_count == 1 || \
		[expr fmod($mom_field_count,2)] == 0.0 } {
		MOM_output_to_listing_device \
		"ERROR: Invalid number of arguments for MCSROT"
		MOM_output_to_listing_device \
		"ACTION: Aborting Coordinate System Rotation"
		set MCSROT_flag 0
		return
		}
		set j 0
		for {set i 1} {$i < 7} {incr i 2} {
		if {[info exists mom_record_fields($i)]} {
			set trans_axis($j) $mom_record_fields($i)
		} else { set trans_axis($j) 0 }
 			set k [expr $i+1]
			if {[info exists mom_record_fields($k)]} {
				set trans_angle($j) $mom_record_fields($k)
			} else { set trans_angle($j) 0 }
				incr j
			}
		}
	}
  	if {[info exists mom_origin]} {
      		set mom_origin_x $mom_origin(0)
      		set mom_origin_y $mom_origin(1)
      		set mom_origin_z $mom_origin(2)
  	}

  	if {[info exists mom_translate]} {
      		set mom_translate_x $mom_translate(0)
      		set mom_translate_y $mom_translate(1)
      		set mom_translate_z $mom_translate(2)
  	}
}

proc GET_TOOL_ATTRIBUTE {} {

	global mom_tool_attribute_text
	global mom_tool_name

	MOM_get_tool_attribute "descp"

	if {$mom_tool_attribute_text == "   "} {
		set mom_tool_attribute_text "NONE"
	}
}

proc CHECK_PLANE_CHANGE {} {
	global mom_motion_type
	global mom_mcs_goto 
	global mom_tool_axis 
	global mom_pos
	global mom_alt_pos 
	global mom_out_angle_pos
	global MCSROT_mode 
	global MCSROT_flag
	global is_MCSROT_matrix_set

	if {$mom_motion_type == "CYCLE" && $MCSROT_mode == "AUTO"} {
      		if {[CHECK_TOOL_AXIS]} {
			MOM_do_template cancel_rotate_mcs
			set MCSROT_flag 0
			CONVERT_CYCLES_DATA
			MOM_do_template rapid_xyz
			set MCSROT_flag 1
			set is_MCSROT_matrix_set 0
		} 
	}
}

proc CHECK_TOOL_AXIS {} {
	global mom_tool_axis
	global mom_prev_tool_axis

	set I [format "%-.7f" $mom_tool_axis(0)]
	set prev_I [format "%-.7f" $mom_prev_tool_axis(0)]
	set J [format "%-.7f" $mom_tool_axis(1)]
	set prev_J [format "%-.7f" $mom_prev_tool_axis(1)]
	set K [format "%-.7f" $mom_tool_axis(2)]
	set prev_K [format "%-.7f" $mom_prev_tool_axis(2)]

	if {$I == $prev_I && $J == $prev_J && $K == $prev_K} {
		return 0
	} else {
		return 1
	}
}

#=============================================================
proc MOM_catch_warning {} {
#=============================================================	
#Begin
#Done
}
#=============================================================
proc Begin {} {
#=============================================================	
	set x [expr [info level]-1]
	MOM_output_literal ">>>>>> Begin: [info level $x]"
}

#=============================================================
proc Done {} {
#=============================================================	
	set x [expr [info level]-1]
	MOM_output_literal "<<<<<< Done: [info level $x]"
	MOM_output_literal "============"
}

#=============================================================
proc say {w} {
#=============================================================	
	MOM_output_literal "  ==== Say: $w ===="
}

#=============================================================
proc MOM_machine_mode {} {
#=============================================================
	MOM_start_of_group
}

#=============================================================
proc MOM_smooth_tool_path { } {
#=============================================================	
	global mom_smooth_path
	
}
#=============================================================
proc MOM_tolerance_cycle { } {
#=============================================================	
	global mom_tolerance_on_off
	global mom_operation_type
	global mom_inside_outside_tolerances
	global tolerances
	global temp
	
	MOM_force once Tol
       
  	if {$mom_operation_type == "Variable-axis Surface Contouring"  	|| 
		$mom_operation_type == "Cavity Milling" 					|| 
		$mom_operation_type == "Fixed-axis Surface Contouring"  	|| 
		$mom_operation_type == "Planar Milling"  				|| 
		$mom_operation_type == "Sequential Mill Main Operation"  	|| 
		$mom_operation_type == "Zig-Zag Surface" } {
  		set temp [expr $mom_inside_outside_tolerances(0) + $mom_inside_outside_tolerances(1)]
  		set tolerances [expr $temp/2]
#  		if {$mom_tolerance_on_off == "ON"} {
#  			MOM_do_template tolerances_1
#  			MOM_do_template tolerances_2
#  		}
  	}
}

#=============================================================
proc MOM_spindle_orient { } {
#=============================================================	
	global mom_angle_orient
	MOM_force once Angle
	MOM_do_template spindle_orient_1
	MOM_do_template spindle_orient_2
}	

#=============================================================
proc  MOM_divide { } {
#=============================================================
	global mom_divide_by mom_length_of_program mom_live_time
	global mom_retract_to mom_offset_dist mom_retract_point_pos
	global max_length 
	
#	MOM_output_literal "DIVIDE"
	set max_length [expr $mom_length_of_program/1.0]
}

#=============================================================
proc  MOM_before_output { } {
#=============================================================
	global mom_o_buffer
}